@@ -25,6 +25,7 @@
 #include "item_strfunc.h"      // Item_str_func
 #include "item_sum.h"
 #include "sql_type_json.h"
+#include "json_schema.h"
 
 class json_path_with_flags
 {
@@ -41,6 +42,13 @@ public:
 };
 
 
+void report_path_error_ex(const char *ps, json_path_t *p,
+                          const char *fname, int n_param,
+                          Sql_condition::enum_warning_level lv);
+void report_json_error_ex(const char *js, json_engine_t *je,
+                          const char *fname, int n_param,
+                          Sql_condition::enum_warning_level lv);
+
 class Json_engine_scan: public json_engine_t
 {
 public:
@@ -53,7 +61,9 @@ public:
                                     (const uchar *) str.end())
   { }
   bool check_and_get_value_scalar(String *res, int *error);
-  bool check_and_get_value_complex(String *res, int *error);
+  bool check_and_get_value_complex(String *res, int *error,
+                                  json_value_types cur_value_type=
+                                                    JSON_VALUE_UNINITIALIZED);
 };
 
 
@@ -76,22 +86,44 @@ protected:
 public:
   Item_func_json_valid(THD *thd, Item *json) : Item_bool_func(thd, json) {}
   longlong val_int() override;
-  const char *func_name() const override { return "json_valid"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
   {
-    if (Item_bool_func::fix_length_and_dec())
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_valid") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    if (Item_bool_func::fix_length_and_dec(thd))
       return TRUE;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
-  bool set_format_by_check_constraint(Send_field_extended_metadata *to) const override
+  bool set_format_by_check_constraint(Send_field_extended_metadata *to) const
+    override
   {
     static const Lex_cstring fmt(STRING_WITH_LEN("json"));
     return to->set_format_name(fmt);
   }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_valid>(thd, this); }
-  enum Functype functype() const override   { return JSON_VALID_FUNC; }
+  enum Functype functype() const override { return JSON_VALID_FUNC; }
+};
+
+
+class Item_func_json_equals: public Item_bool_func
+{
+public:
+  Item_func_json_equals(THD *thd, Item *a, Item *b):
+    Item_bool_func(thd, a, b) {}
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_equals") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_equals>(thd, this); }
+  longlong val_int() override;
 };
 
 
@@ -104,8 +136,12 @@ protected:
 public:
   Item_func_json_exists(THD *thd, Item *js, Item *i_path):
     Item_bool_func(thd, js, i_path) {}
-  const char *func_name() const override { return "json_exists"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_exists") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_exists>(thd, this); }
   longlong val_int() override;
@@ -137,8 +173,12 @@ class Item_func_json_value: public Item_str_func,
 public:
   Item_func_json_value(THD *thd, Item *js, Item *i_path):
     Item_str_func(thd, js, i_path) {}
-  const char *func_name() const override { return "json_value"; }
-  bool fix_length_and_dec() override ;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_value") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override ;
   String *val_str(String *to) override
   {
     null_value= Json_path_extractor::extract(to, args[0], args[1],
@@ -161,8 +201,12 @@ class Item_func_json_query: public Item_json_func,
 public:
   Item_func_json_query(THD *thd, Item *js, Item *i_path):
     Item_json_func(thd, js, i_path) {}
-  const char *func_name() const override { return "json_query"; } 
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_query") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *to) override
   {
     null_value= Json_path_extractor::extract(to, args[0], args[1],
@@ -172,7 +216,7 @@ public:
   bool check_and_get_value(Json_engine_scan *je,
                            String *res, int *error) override
   {
-    return je->check_and_get_value_complex(res, error);
+    return je->check_and_get_value_complex(res, error, JSON_VALUE_UNINITIALIZED);
   }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_query>(thd, this); }
@@ -186,8 +230,12 @@ protected:
 
 public:
   Item_func_json_quote(THD *thd, Item *s): Item_str_func(thd, s) {}
-  const char *func_name() const override { return "json_quote"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_quote") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_quote>(thd, this); }
@@ -201,8 +249,12 @@ protected:
   String *read_json(json_engine_t *je);
 public:
   Item_func_json_unquote(THD *thd, Item *s): Item_str_func(thd, s) {}
-  const char *func_name() const override { return "json_unquote"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_unquote") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_unquote>(thd, this); }
@@ -247,9 +299,13 @@ public:
                     char **out_val, int *value_len);
   Item_func_json_extract(THD *thd, List<Item> &list):
     Item_json_str_multipath(thd, list) {}
-  const char *func_name() const override { return "json_extract"; }
-  enum Functype functype() const override   { return JSON_EXTRACT_FUNC; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_extract") };
+    return name;
+  }
+  enum Functype functype() const override { return JSON_EXTRACT_FUNC; }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   longlong val_int() override;
   double val_real() override;
@@ -271,8 +327,12 @@ protected:
 public:
   Item_func_json_contains(THD *thd, List<Item> &list):
     Item_bool_func(thd, list) {}
-  const char *func_name() const override { return "json_contains"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_contains") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   longlong val_int() override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_contains>(thd, this); }
@@ -293,9 +353,13 @@ public:
   Item_func_json_contains_path(THD *thd, List<Item> &list):
     Item_bool_func(thd, list), tmp_paths(0) {}
   virtual ~Item_func_json_contains_path();
-  const char *func_name() const override { return "json_contains_path"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_contains_path") };
+    return name;
+  }
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   longlong val_int() override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_contains_path>(thd, this); }
@@ -313,8 +377,12 @@ public:
   Item_func_json_array(THD *thd, List<Item> &list):
     Item_json_func(thd, list) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
-  const char *func_name() const override { return "json_array"; }
+  bool fix_length_and_dec(THD *thd) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_array") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_array>(thd, this); }
 };
@@ -328,10 +396,14 @@ protected:
 public:
   Item_func_json_array_append(THD *thd, List<Item> &list):
     Item_json_str_multipath(thd, list) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   uint get_n_paths() const override { return arg_count/2; }
-  const char *func_name() const override { return "json_array_append"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_array_append") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_array_append>(thd, this); }
 };
@@ -343,7 +415,11 @@ public:
   Item_func_json_array_insert(THD *thd, List<Item> &list):
     Item_func_json_array_append(thd, list) {}
   String *val_str(String *) override;
-  const char *func_name() const override { return "json_array_insert"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_array_insert") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_array_insert>(thd, this); }
 };
@@ -357,7 +433,11 @@ public:
   Item_func_json_object(THD *thd, List<Item> &list):
     Item_func_json_array(thd, list) {}
   String *val_str(String *) override;
-  const char *func_name() const override { return "json_object"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_object") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_object>(thd, this); }
 };
@@ -371,7 +451,11 @@ public:
   Item_func_json_merge(THD *thd, List<Item> &list):
     Item_func_json_array(thd, list) {}
   String *val_str(String *) override;
-  const char *func_name() const override { return "json_merge_preserve"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_merge_preserve") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_merge>(thd, this); }
 };
@@ -381,24 +465,65 @@ class Item_func_json_merge_patch: public Item_func_json_merge
 public:
   Item_func_json_merge_patch(THD *thd, List<Item> &list):
     Item_func_json_merge(thd, list) {}
-  const char *func_name() const override { return "json_merge_patch"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_merge_patch") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_merge_patch>(thd, this); }
 };
 
+
+class Item_func_json_normalize: public Item_json_func
+{
+public:
+  Item_func_json_normalize(THD *thd, Item *a):
+    Item_json_func(thd, a) {}
+  String *val_str(String *) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_normalize") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_normalize>(thd, this); }
+};
+
+
+class Item_func_json_object_to_array: public Item_json_func
+{
+  protected:
+  String tmp;
+public:
+  Item_func_json_object_to_array(THD *thd, Item *a):
+    Item_json_func(thd, a) {}
+  String *val_str(String *) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_object_to_array") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_object_to_array>(thd, this); }
+};
+
+
 class Item_func_json_length: public Item_long_func
 {
   bool check_arguments() const override
   {
+    const LEX_CSTRING name= func_name_cstring();
     if (arg_count == 0 || arg_count > 2)
     {
-      my_error(ER_WRONG_PARAMCOUNT_TO_NATIVE_FCT, MYF(0), func_name());
+      my_error(ER_WRONG_PARAMCOUNT_TO_NATIVE_FCT, MYF(0), name.str);
       return true;
     }
-    return args[0]->check_type_can_return_text(func_name()) ||
-           (arg_count > 1 &&
-            args[1]->check_type_general_purpose_string(func_name()));
+    return args[0]->check_type_can_return_text(name) ||
+      (arg_count > 1 && args[1]->check_type_general_purpose_string(name));
   }
 protected:
   json_path_with_flags path;
@@ -407,8 +532,12 @@ protected:
 public:
   Item_func_json_length(THD *thd, List<Item> &list):
     Item_long_func(thd, list) {}
-  const char *func_name() const override { return "json_length"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_length") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   longlong val_int() override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_length>(thd, this); }
@@ -418,13 +547,17 @@ public:
 class Item_func_json_depth: public Item_long_func
 {
   bool check_arguments() const override
-  { return args[0]->check_type_can_return_text(func_name()); }
+  { return args[0]->check_type_can_return_text(func_name_cstring()); }
 protected:
   String tmp_js;
 public:
   Item_func_json_depth(THD *thd, Item *js): Item_long_func(thd, js) {}
-  const char *func_name() const override { return "json_depth"; }
-  bool fix_length_and_dec() override { max_length= 10; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_depth") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override { max_length= 10; return FALSE; }
   longlong val_int() override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_depth>(thd, this); }
@@ -437,8 +570,12 @@ protected:
   String tmp_js;
 public:
   Item_func_json_type(THD *thd, Item *js): Item_str_func(thd, js) {}
-  const char *func_name() const override { return "json_type"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_type") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_type>(thd, this); }
@@ -455,13 +592,16 @@ public:
   Item_func_json_insert(bool i_mode, bool r_mode, THD *thd, List<Item> &list):
     Item_json_str_multipath(thd, list),
       mode_insert(i_mode), mode_replace(r_mode) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   uint get_n_paths() const override { return arg_count/2; }
-  const char *func_name() const override
+  LEX_CSTRING func_name_cstring() const override
   {
-    return mode_insert ?
-             (mode_replace ? "json_set" : "json_insert") : "json_replace";
+    static LEX_CSTRING json_set=    {STRING_WITH_LEN("json_set") };
+    static LEX_CSTRING json_insert= {STRING_WITH_LEN("json_insert") };
+    static LEX_CSTRING json_replace= {STRING_WITH_LEN("json_replace") };
+    return (mode_insert ?
+            (mode_replace ? json_set : json_insert) : json_replace);
   }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_insert>(thd, this); }
@@ -475,10 +615,14 @@ protected:
 public:
   Item_func_json_remove(THD *thd, List<Item> &list):
     Item_json_str_multipath(thd, list) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   uint get_n_paths() const override { return arg_count - 1; }
-  const char *func_name() const override { return "json_remove"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_remove") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_remove>(thd, this); }
 };
@@ -493,8 +637,12 @@ protected:
 public:
   Item_func_json_keys(THD *thd, List<Item> &list):
     Item_str_func(thd, list) {}
-  const char *func_name() const override { return "json_keys"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_keys") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_json_keys>(thd, this); }
@@ -516,9 +664,13 @@ protected:
 public:
   Item_func_json_search(THD *thd, List<Item> &list):
     Item_json_str_multipath(thd, list) {}
-  const char *func_name() const override { return "json_search"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_search") };
+    return name;
+  }
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *) override;
   uint get_n_paths() const override { return arg_count > 4 ? arg_count - 4 : 0; }
   Item *do_get_copy(THD *thd) const override
@@ -545,8 +697,8 @@ public:
   Item_func_json_format(THD *thd, List<Item> &list):
     Item_json_func(thd, list), fmt(DETAILED) {}
 
-  const char *func_name() const override;
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str(String *str) override;
   String *val_json(String *str) override;
   Item *do_get_copy(THD *thd) const override
@@ -584,7 +736,11 @@ public:
     return Type_handler_json_common::json_type_handler_sum(this);
   }
 
-  const char *func_name() const override { return "json_arrayagg("; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_arrayagg(") };
+    return name;
+  }
   bool fix_fields(THD *thd, Item **ref) override;
   enum Sumfunctype sum_func() const override { return JSON_ARRAYAGG_FUNC; }
 
@@ -604,14 +760,18 @@ public:
     Item_sum(thd, key, value)
   {
     quick_group= FALSE;
-    result.append("{");
+    result.append('{');
   }
 
   Item_func_json_objectagg(THD *thd, Item_func_json_objectagg *item);
   void cleanup() override;
 
-  enum Sumfunctype sum_func() const override {return JSON_OBJECTAGG_FUNC;}
-  const char *func_name() const override { return "json_objectagg"; }
+  enum Sumfunctype sum_func () const override { return JSON_OBJECTAGG_FUNC;}
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_objectagg") };
+    return name;
+  }
   const Type_handler *type_handler() const override
   {
     return Type_handler_json_common::json_type_handler_sum(this);
@@ -642,4 +802,142 @@ public:
 
 extern bool is_json_type(const Item *item);
 
+class Item_func_json_overlaps: public Item_bool_func
+{
+  String tmp_js;
+  bool a2_constant, a2_parsed;
+  String tmp_val, *val;
+public:
+  Item_func_json_overlaps(THD *thd, Item *a, Item *b):
+    Item_bool_func(thd, a, b)
+    {}
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_overlaps") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  longlong val_int() override;
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_overlaps>(thd, this); }
+};
+
+class Item_func_json_schema_valid: public Item_bool_func
+{
+  String tmp_js;
+  bool schema_parsed;
+  String tmp_val, *val;
+  List<Json_schema_keyword> keyword_list;
+  List<Json_schema_keyword> all_keywords;
+
+public:
+  Item_func_json_schema_valid(THD *thd, Item *a, Item *b):
+    Item_bool_func(thd, a, b)
+    {
+      val= NULL;
+      schema_parsed= false;
+      set_maybe_null();
+    }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_schema_valid") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  longlong val_int() override;
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_schema_valid>(thd, this); }
+  void cleanup() override;
+};
+
+class Item_func_json_key_value: public Item_json_func,
+                            public Json_path_extractor
+{
+
+  String tmp_str;
+
+public:
+  Item_func_json_key_value(THD *thd, Item *js, Item *i_path):
+    Item_json_func(thd, js, i_path) {}
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_key_value") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
+  String *val_str(String *to) override;
+  bool check_and_get_value(Json_engine_scan *je,
+                           String *res, int *error) override
+  {
+    return je->check_and_get_value_complex(res, error, JSON_VALUE_OBJECT);
+  }
+  bool get_key_value(json_engine_t *je, String *str);
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_key_value>(thd, this); }
+};
+
+
+
+class Item_func_json_array_intersect: public Item_str_func
+{
+protected:
+  String tmp_js1, tmp_js2;
+  bool hash_inited, root_inited;
+  HASH items;
+  MEM_ROOT hash_root;
+  bool parse_for_each_row;
+public:
+  Item_func_json_array_intersect(THD *thd, Item *a, Item *b):
+    Item_str_func(thd, a, b) { hash_inited= root_inited= parse_for_each_row= false; }
+  String *val_str(String *) override;
+  bool fix_length_and_dec(THD *thd) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_array_intersect") };
+    return name;
+  }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_array_intersect>(thd, this); }
+  void cleanup() override
+  {
+    Item_str_func::cleanup();
+    if (hash_inited)
+      my_hash_free(&items);
+    if (root_inited)
+      free_root(&hash_root, MYF(0));
+  }
+  void prepare_json_and_create_hash(json_engine_t *je1, String *js);
+};
+
+class Item_func_json_object_filter_keys: public Item_str_func
+{
+protected:
+  String tmp_js1, tmp_js2;
+  bool hash_inited, root_inited;
+  HASH items;
+  MEM_ROOT hash_root;
+public:
+  Item_func_json_object_filter_keys(THD *thd, Item *a, Item *b):
+    Item_str_func(thd, a, b) { hash_inited= root_inited= false; }
+  String *val_str(String *) override;
+  bool fix_length_and_dec(THD *thd) override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("json_object_filter_keys") };
+    return name;
+  }
+  Item *do_get_copy(THD *thd) const override
+  { return get_item_copy<Item_func_json_object_filter_keys>(thd, this); }
+
+  void cleanup() override
+  {
+    Item_str_func::cleanup();
+    if (hash_inited)
+      my_hash_free(&items);
+    if (root_inited)
+      free_root(&hash_root, MYF(0));
+  }
+};
+
+
 #endif /* ITEM_JSONFUNC_INCLUDED */
